using System.ComponentModel;
using System.Diagnostics;
using System.Net;
using System.Text.Json;
using GraphicsManager;
using GraphicsManager.Enums;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using OpenTK.Windowing.Desktop;

namespace Updater;

public class NewUpdater : FPSWindow
{
    private readonly ConfigFile cf;
    private Texture ProgressbarTexture;
    private ProgressBar TotalProgress;
    private Argumets Argumets;
    private List<ProgressBar> ProgressThreads = new();
    public FontInteraction fi;
    private int WallDistance;
    private readonly BackgroundWorker Download = new();
    public NewUpdater(NativeWindowSettings nws, GameWindowSettings gws, Argumets args)
    :base(nws, gws)
    {
        cf = Config.GetConfig();
        Argumets = args;
        fi = FontInteraction.Load(FontFamily.LoadFontFamily("Noto Sans").Result);
        BackgroundColor = cf.Colors.Background.ToColor4();
        WallDistance = cf.Scale(16);
        ProgressbarTexture = TextureManager.AddTexture(File.OpenRead("/home/jacob/Pictures/Progress.png"));
        Controls.Add(TotalProgress = new ProgressBar(ProgressbarTexture)
        {
            Location = new(WallDistance, cf.Scale(70), 0),
            Size = new(Size.X - WallDistance - WallDistance, cf.Scale(20)),
            BackgroundColor = cf.Colors.Progress_bars.Backcolor.ToColor4(),
            ProgressColor = cf.Colors.Progress_bars.Fillcolor.ToColor4(),
            ProgressValue = 25,
            ProgressGap = 4,
            Shader = Rectangle.DefaultAlphaShader[Context],
            TextureDisplay = TextureDisplay.ProgressHorizontalCenter,
            UpdateOnDraw = true
        });
        TotalProgress.InnerShader = TotalProgress.Shader;
        int gap = cf.Scale(15);
        for (int i = 0; i < cf.Format.DownloadThreads; i++)
        {
            ProgressBar temp;
            Size = new(Size.X, Size.Y + gap + TotalProgress.Size.Y);
            Controls.Add(temp = new ProgressBar(ProgressbarTexture)
            {
                Location = new(WallDistance, cf.Scale(70) + (TotalProgress.Size.Y * (i + 1)) + (cf.Scale(15) * (i + 1)), 0),
                Size = TotalProgress.Size,
                BackgroundColor = TotalProgress.BackgroundColor,
                ProgressColor = TotalProgress.ProgressColor,
                ProgressValue = 100,
                ProgressGap = TotalProgress.ProgressGap,
                Shader = TotalProgress.Shader,
                TextureDisplay = TotalProgress.TextureDisplay,
                UpdateOnDraw = true
            });
            temp.InnerShader = TotalProgress.Shader;
            ProgressThreads.Add(temp);
        }
        fi.PixelHeight = (uint)cf.Scale(15);
        Download.DoWork += new DoWorkEventHandler(Download_DoWork);
        Download.RunWorkerCompleted += Download_RunWorkerCompleted;
        Download.WorkerReportsProgress = true;
        Invoke(new Action(() => { Download.RunWorkerAsync(); }));
        
        
        
        Controls.Add(new Label(fi)
        {
            Location = new(10,10,0),
            Text = FloatToInt(-0.11304348f, true).ToString() + " / " 
                + FloatToInt(-0.133858263f, true).ToString() + " / "
                + FloatToInt(-0.314960629f, true).ToString() + " / "
                + FloatToInt(-0.341389941f, true).ToString() + " / "
                + FloatToInt(-0.5748032f, true).ToString()
        });
    }
    
    private void Download_DoWork(object? sender, DoWorkEventArgs e)
    {
        try
        {
            bool goodpre = true;
            while (goodpre)
            {
                while (!Program.IsProcessOpen(Argumets.Process))
                {
                    goodpre = false;
                    break;
                }
            }
            ChangeProcessText("Deleting old program files");
            
            void TryDeleteDir(string? dir)
            {
                if (string.IsNullOrEmpty(dir) || string.IsNullOrWhiteSpace(dir)) return;
                try
                {
                    if (Directory.Exists(dir)) Directory.Delete(dir, true);
                }
                catch
                {
                    DirectoryInfo dirInfo = new(dir);
                    foreach (var file in dirInfo.GetFiles())
                    {
                        try
                        {
                            file.Delete();
                        }
                        catch { }
                    }
                    foreach (var subdir in dirInfo.GetDirectories())
                    {
                        TryDeleteDir(subdir.FullName);
                    }
                }
            }
            TryDeleteDir(Argumets.LocalDirectory);
            Directory.CreateDirectory(Argumets.LocalDirectory);
            if (Argumets.Updater is not null) TryDeleteDir(Argumets.Updater);
            if (Argumets.Updater is not null) Directory.CreateDirectory(Argumets.Updater);

    #pragma warning disable SYSLIB0014 // Type or member is obsolete
            using WebClient webPre = new();
    #pragma warning restore SYSLIB0014 // Type or member is obsolete
            ChangeProcessText("Getting update info");
            string rem = webPre.DownloadString($"https://{Program.Domain}/Updater/Files?directory={Argumets.RemoteDirectory}{Argumets.UriBranch}{Argumets.UriSeflContaind}{Argumets.UriPlatform}{Argumets.UriVersion}");
            ulong realmax;
            string uri;
            try
            {
                realmax = ulong.Parse(webPre.DownloadString($"https://{Program.Domain}/Updater/GetSize?directory={Argumets.RemoteDirectory}{Argumets.UriBranch}{Argumets.UriSeflContaind}{Argumets.UriPlatform}{Argumets.UriVersion}"));
                TotalProgress.MaxProgressValue = realmax;
                Console.WriteLine(TotalProgress.MaxProgressValue);
                //queue.Enqueue(new Action(() => { if (Total_Downloaded is not null) Total_Downloaded.Text = TotalDownloadAmountNeat(realmax); }));
                //Speed.Start();
                string[] files = rem.Split('\n');
                int c = -1;

                ProgressBar getpb()
                {
                    c++;
                    if (c == ProgressThreads.Count) c = 0;
                    return ProgressThreads[c];
                }
                _ = Parallel.ForEach(files, new ParallelOptions() {MaxDegreeOfParallelism = cf.Format.DownloadThreads},file =>
                {
#pragma warning disable SYSLIB0014 // Type or member is obsolete
                    using WebClient web = new();
#pragma warning restore SYSLIB0014 // Type or member is obsolete
                    
                    if (string.IsNullOrEmpty(file)) return;
                    Console.WriteLine(file);
                    uri = $"https://{Program.Domain}/Updater/GetFileSize?directory={Argumets.RemoteDirectory}{Argumets.UriBranch}{Argumets.UriVersion}{Argumets.UriSeflContaind}{Argumets.UriPlatform}&file={file}";
                    Uri u = new($"https://{Program.Domain}/Updater/GetFile?directory={Argumets.RemoteDirectory}{Argumets.UriBranch}{Argumets.UriVersion}{Argumets.UriSeflContaind}{Argumets.UriPlatform}&file={file}");
                    Console.WriteLine(file);
                    Console.WriteLine(uri);
                    string[] temp = file.Split('/');
                    if (file.Contains('/'))
                    {

                        string cur = "";
                        for (int i = 0; i < temp.Length - 1; i++)
                        {
                            cur += temp[i] + "/";
                            if (Argumets.Updater is null)
                            {
                                if (!Directory.Exists(Argumets.LocalDirectory + cur))
                                    Directory.CreateDirectory(Argumets.LocalDirectory + cur);
                            }
                            else
                            {
                                if (!Directory.Exists(Argumets.Updater + cur))
                                    Directory.CreateDirectory(Argumets.Updater + cur);
                            }
                        }
                    }
                    
                   
                    DownloadFile(getpb(), web, uri, u, temp, file.Replace("/", Program.Slash)).Wait();
                });
            }
            catch (Exception ex)
            {
                StackTrace st = new(ex, true);
                StackFrame? frame = st.GetFrame(0);
                if (frame is null) return;
                int line = frame.GetFileLineNumber();
                string msg = $"Error in Updater.exe in function Download_DoWork on line '{line}' through error '{ex.Message}'\nERROR:\n'{ex}' FRAM:\n '{frame}'";
                MessageBox.Show(ex.ToString());
                _ = new HttpClient().PostAsync($"https://{Program.Domain}/Updater/Error/updater", new StringContent(msg));
            }
        }
        catch (Exception exception)
        {
            Console.WriteLine(exception);
            throw;
        }
    }
    
    private Task DownloadFile(ProgressBar pb, WebClient web, string uri, Uri u, string[] temp, string file)
    {
        while (!web.IsBusy)
        {
            string a = web.DownloadString(uri);
            //Invoke(new Action(() =>pb.MaxProgressValue = ulong.Parse(a)));
            pb.MaxProgressValue = ulong.Parse(a);
            Console.WriteLine(uri + ": " +pb.MaxProgressValue);
            //ulong byres = 0;
            web.DownloadProgressChanged += (sender, args) =>
            {
                TotalProgress.ProgressValue += (ulong)args.BytesReceived - pb.ProgressValue;
                pb.ProgressValue = (ulong)args.BytesReceived;
            };
            ChangeProcessText($"Downloading {temp[^1]}");
            if (Argumets.Updater is null)
            {
                web.DownloadFileTaskAsync(u, Argumets.LocalDirectory + file).Wait();
            }
            else
            {
                web.DownloadFileTaskAsync(u, Argumets.Updater + file).Wait();
            }

            return Task.CompletedTask;
        }
        return Task.CompletedTask;
    }
    
    private void Download_RunWorkerCompleted(object? sender, RunWorkerCompletedEventArgs e)
    {
        try
        {
            if (Argumets.Updater is not null)
            {
                Process p = new();
                p.StartInfo.FileName = Path.Combine(Argumets.Updater, AppDomain.CurrentDomain.FriendlyName);
                if (OperatingSystem.IsLinux() || OperatingSystem.IsMacOS() || OperatingSystem.IsFreeBSD())
                {
                    Process m = Process.Start("/usr/bin/chmod", $"+x {Path.Combine(Argumets.Updater, AppDomain.CurrentDomain.FriendlyName)}");
                    m.WaitForExit();
                }

                p.StartInfo.WorkingDirectory = Argumets.Updater;
                p.StartInfo.CreateNoWindow = true;
                Program.rawargs[1] = AppDomain.CurrentDomain.BaseDirectory;
                p.StartInfo.Arguments = $"\"{string.Join("\" \"", Program.rawargs)}\"";
                p.StartInfo.WindowStyle = ProcessWindowStyle.Normal;
            
                p.Start();
                Console.WriteLine("Program done");
                Close();
            }
            else
            {
                if (Argumets.Loc is not null)
                {
                    FileInfo fi = new(Argumets.Loc);
                    if (!fi.Directory!.Exists) Directory.CreateDirectory(fi.Directory.FullName);
                    UpdaterSettings temp;
                    if (!File.Exists(Argumets.Loc))
                    {
                        File.WriteAllText(Argumets.Loc, JsonSerializer.Serialize(new UpdaterSettings(), UpdaterSettingsContext.Default.UpdaterSettings));
                        temp = new();
                    }

                    try
                    {
                        UpdaterSettings? ss = JsonSerializer.Deserialize(File.ReadAllText(Argumets.Loc), UpdaterSettingsContext.Default.UpdaterSettings);
                        if (ss is null)
                        {
                            ss = new();
                        }

                        temp = ss;
                    }
                    catch
                    {
                        temp = new();
                    }

                    temp.Updater = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, AppDomain.CurrentDomain.FriendlyName);
                    if (Argumets.Platform is not null) temp.Platform = Argumets.Platform!.ToLower();
                    temp.SelfContained = Argumets.SeflContaind;
                    temp.Branch = Argumets.Branch!.ToLower() switch
                    {
                        "dev" => Branch.Dev,
                        "beta" => Branch.Beta,
                        "master" or _ => Branch.Master
                    };
                    File.WriteAllText(Argumets.Loc, JsonSerializer.Serialize(temp, UpdaterSettingsContext.Default.UpdaterSettings));
                }
                if (string.IsNullOrEmpty(Argumets.DLL) || string.IsNullOrWhiteSpace(Argumets.DLL)) Close();
                Process p = new();
                p.StartInfo.FileName = Path.Combine(Argumets.LocalDirectory, Argumets.DLL);
                if (OperatingSystem.IsLinux() || OperatingSystem.IsMacOS() || OperatingSystem.IsFreeBSD())
                {
                    Process m = Process.Start("/usr/bin/chmod", $"+x \"{Path.Combine(Argumets.LocalDirectory, Argumets.DLL)}\"");
                    m.WaitForExit();
                }

                p.StartInfo.WorkingDirectory = Argumets.LocalDirectory;
                p.StartInfo.CreateNoWindow = true;
                foreach (string arg in Argumets.args!)
                {
                    p.StartInfo.ArgumentList.Add(arg);
                }

                p.StartInfo.WindowStyle = ProcessWindowStyle.Normal;

                p.Start();
                Close();
            }
        }
        catch (Exception exception)
        {
            Console.WriteLine(exception);
        }
    }
    
    private void ChangeProcessText(string Process)
    {
        Console.WriteLine(Process);
        //queue.Enqueue(new Action(() => { if (Current_Process is not null) Current_Process.Text = $"{Process}"; }));
    }
}